require 'test/unit'
require '../lib/BotnetV2'

class MasterTest < Test::Unit::TestCase

  # Called before every test method runs. Can be used
  # to set up fixture information.
  def setup
    # Do nothing
    unless File.exists? 'private.pem' and File.exists? 'cert.pem'
      File.delete 'private.pem' if File.exists? 'private.pem'
      File.delete 'cert.pem' if File.exists? 'cert.pem'
      BotnetV2::BotnetV2.create_certificate
    end
  end

  # Called after every test method runs. Can be used to tear
  # down fixture information.

  def teardown
    # Do nothing
  end

  def test_create
    # Create a master and a client to play with.
    master = BotnetV2::BotnetV2.create_master 'foobar'
    client = BotnetV2::BotnetV2.create_client 'foobar', 1, 'localhost'
    @verified_clients = 1
    @ready_workers = 0
    @processed_tasks = 0

    # Test status of botnet...
    client.net_status(Proc.new do |res| on_status(res) end)
    sleep(0) until @done
    @done = false

    # Create a worker
    worker = BotnetV2::BotnetV2.create_worker 'foobar', 'localhost'
    @verified_clients = 2
    @ready_workers = 1

    # Now execute tasks and watch the status of the server change.
    client.execute('1+1', Proc.new do |res| on_result(res) end)
    sleep(0) until @done
    @processed_tasks = 1
    @done = false
    client.net_status(Proc.new do |res| on_status(res) end)
    sleep(0) until @done
    @done = false
    client.execute('1+1', Proc.new do |res| on_result(res) end)
    sleep(0) until @done
    @processed_tasks = 2
    @done = false
    client.net_status(Proc.new do |res| on_status(res) end)
    sleep(0) until @done
    client.exit!
    worker.exit!
    master.exit!
  end

  def on_status(result)
    assert_equal(result['ready_clients'], 1, "Invalid client count.")
    assert_equal(result['verified_clients'], @verified_clients, "Invalid verified client count.")
    assert_equal(result['ready_workers'], @ready_workers, "Invalid worker count.")
    assert_equal(result['work_bundles'], 0, "Invalid worker count.")
    assert_equal(result['processed_tasks'], @processed_tasks, "Invalid amount of tasks processed.")
    assert_equal(result['active_tasks'], 0, "Invalid amount of tasks active.")
    @done = true
  end

  def on_result(result)
    assert_equal(result, 2, 'Wrong result')
    @done = true
  end
end